--[[
TCFT3
Event handlers
]]

TCFT3.Events = TCFT3.Events or {}

-- Nop - dummy command from server
TCFT3.Events[9998] = function()
	-- NOP - user not logged in
end

-- Login OK - update everything
TCFT3.Events[500] = function(params)
	TCFT3.Connection.isConnected = true
	TCFT3.Connection.isLoggedin = true
	-- Check if client version is current
	TCFT3.SendData({ cmd="1000", version=TCFT3.Version })
	TCFT3.connectButton.title = "Disconnect"
	TCFT3.SetStatusLine(TCFT3.colors.GREEN2 .. "Connected to the server.")
	TCFT3.PloziNav.Initialize() -- Initialize navigation computer
	TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.NORMAL .. "Auto Pilot Enabled", true)
	TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) *** Currently logged on users: " .. params.users, true)
	TCFT3.UserOptions = params.useroptions
	TCFT3.SendData({ cmd="1500" })
	if (tonumber(TCFT3.UserOptions.memberedit)==1) then -- We can edit members
		TCFT3.Guild.MemberEditButtons.visible = "YES"
	end
	if (tonumber(TCFT3.UserOptions.navedit)==1) then -- We can edit navroutes
		TCFT3.PloziNav.UI.SendToServerBtn.active = "YES"
	end
	
	-- Activate buttons that might have been disabled
	if (TCFT3.Settings.Data.autoloadsectordata=="ON") then -- Auto load sector data
		TCFT3.Mining.LoadSector()
	end
	-- Display guild members
	if (TCFT3.Settings.Data.displayguildlog=="ON") then
		TCFT3.GuildOnlineTimer:SetTimeout(5000, TCFT3.ShowOnlineGuildMembers)
	end
end

-- Invalid login
TCFT3.Events[501] = function(params)
	TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.RED .. "Login to server failed.", true)
	TCFT3.Connection.Socket:Disconnect()
	TCFT3.Connection.Socket = nil
	TCFT3.Connection.isConnected = false
	TCFT3.Connection.isLoggedin = false
	TCFT3.connectButton.title = " Connect "
	TCFT3.SetStatusLine(TCFT3.colors.RED .. "Not Connected to the server.")
	TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.RED .. "Connection to server closed.", true)
end

-- Notify that other user logged in
TCFT3.Events[502] = function(params)
local user = params.username
TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. string.format("%s logged in.", params.username) .. TCFT3.colors.GREEN2, true)
end
-- 
-- -- Notify that other user logged out
TCFT3.Events[503] = function(params)
local user = params.username or "This shouldnt occour... "
TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. string.format("%s logged out.", params.username) .. TCFT3.colors.GREEN2, true)
end

-- Not compatible versions
TCFT3.Events[505] = function(params)
	--TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.RED .. "Your clientversion are not compatible with the current server version.", true)
	--TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.NORMAL .. "To be able to continue using TCFT3 you have to upgrade to latest version.", true)
	--TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.NORMAL .. "You can download at http://vo.aoczone.net/download", true)
end

-- Reveiving list of logged in users
TCFT3.Events[550] = function(params)
	TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. "*** Currently logged on users: " .. params.users .. TCFT3.colors.NORMAL)
end

-- TCFT chat
TCFT3.Events[600] = function(params)
	local msg = params.msg
	if (string.len(msg)>0) then
		local sectorid = tonumber(params.sectorid) or -1
		local locstr = " "
		if (sectorid>-1) then
			locstr = string.format(" [%s]", ShortLocationStr(sectorid))
		end
		local name = params.name or "Unknown"
		local isemote = params.isemote or 0
		local fullmsg
		if (isemote==1) then
			fullmsg = string.format(TCFT3.colors.GREEN2 .. "(TCFT3)%s **%s%s%s %s", locstr, TCFT3.colors.NORMAL, name, TCFT3.colors.GREEN2, msg)
		else
			fullmsg = string.format(TCFT3.colors.GREEN2 .. "(TCFT3)%s <%s%s%s> %s", locstr, TCFT3.colors.NORMAL, name, TCFT3.colors.GREEN2, msg)
		end
		TCFT3.AddStatus(fullmsg)
		if (TCFT3.Settings.Data.tcftchattomainchat=="ON") then
			print(fullmsg)
		end
	end
end

-- Send some lines from tradebuffer
TCFT3.Events[700] = function(params)
	local buffer = {}
	for i=1,25 do
		local item = table.remove(TCFT3.Trading.TradeBuffer)
		if (item==nil) then break end
		table.insert(buffer, item)
	end
	if (#buffer>0) then
		TCFT3.SendData({ cmd="301", items=buffer })
	else
		TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.NORMAL .. "Alts submited to TGFT Commander.", true)
	end
end

-- Update tradeitemnames in trade tab
TCFT3.Events[710] = function(params)
	TCFT3.Trading.Itemlist = params.items
	TCFT3.Trading.UpdateLookupItems()
	TCFT3.Trading.LastUpdate = os.time()
end

local function tableSort(a, b)
	return a.name<b.name
end

-- Update stationnames
TCFT3.Events[730] = function(params)
	local num = tonumber(params.numitems)
	if (num>0) then
		TCFT3.Stations = {}
		for _, station in ipairs(params.stations) do
			local stationid = station.id
			local sectorid = SplitStationID(stationid)
			local sectname = ShortLocationStr(sectorid)
			table.insert(TCFT3.Stations, { id=stationid, name=sectname .. " - " .. GetStationName(stationid) })
		end
	end
	for id, name in pairs(SystemNames) do
		local num = tonumber(id) or -1
		if (num>-1) then
			table.insert(TCFT3.Stations, { id=-num, name=name .. " (System)" })
		end
	end
	table.sort(TCFT3.Stations, tableSort)
	TCFT3.Trading.StationCompareUpdateStations()
	TCFT3.Trading.TradeRouteSearchUpdateStations()
end

-- Item lookup response
TCFT3.Events[750] = function(params)
	TCFT3.Trading.Lookup.Result = {}
	local num = tonumber(params.numitems)
	if (num>0) then
		for _, item in ipairs(params.items) do
			table.insert(TCFT3.Trading.Lookup.Result, item)
		end
	end
	TCFT3.Trading.Lookup.UpdateLookupTable()
end

-- Compare stations response
TCFT3.Events[760] = function(params)
	if (TCFT3.Trading.StationCompareCancelled==false) then
		ProcessEvent("TCFT3_UPDATEPROGRESSBAR", params.progress)
		local num = tonumber(params.numitems)
		if (num>0) then
			for _, item in ipairs(params.items) do
				item.type = TCFT3.itemtypesrev[tonumber(item.type)] or "???"
				table.insert(TCFT3.Trading.StationCompareResult, item)
			end
			TCFT3.SendData({ cmd="471" })
		else
			ProcessEvent("TCFT3_PROGRESSBARDONE")
			TCFT3.Trading.UpdateStationCompareTable()
		end
	else
		TCFT3.SendData({ cmd="472" }) -- Tell server we dont need more data
		TCFT3.Trading.UpdateStationCompareTable()
	end
end

-- Trade Route Search Result
TCFT3.Events[770] = function(params)
	if (TCFT3.Trading.TradeRouteSearchCancelled==false) then
		ProcessEvent("TCFT3_UPDATEPROGRESSBAR", params.progress)
		local num = tonumber(params.numitems)
		if (num>0) then
			for _, item in ipairs(params.items) do
				-- item.type = TCFT3.itemtypesrev[tonumber(item.type)] or "???"
				table.insert(TCFT3.Trading.TradeRouteSearchResult, item)
			end
			TCFT3.SendData({ cmd="481" })
		else
			ProcessEvent("TCFT3_PROGRESSBARDONE")
			TCFT3.Trading.UpdateTradeRouteList()
		end
	else
		TCFT3.SendData({ cmd="482" })
		TCFT3.Trading.UpdateTradeRouteList()
	end
end

-- Submit some sector data
TCFT3.Events[800] = function(params)
	local buffer = {}
	for i=1,10 do
		local item = table.remove(TCFT3.Mining.RoidBuffer)
		if (item==nil) then break end
		table.insert(buffer, item)
	end
	if (#buffer>0) then
		TCFT3.SendData({ cmd="501", items=buffer })
	else
		TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.NORMAL .. "Sector submitted.", true)
		-- Load system AFTER submitted current one
		if (TCFT3.Settings.Data.autoloadsectordata=="ON") then -- Auto load sector data
			--TCFT3.Mining.LoadSystem()
		end
	end
end

-- Reveiving orelist - update 
TCFT3.Events[810] = function(params)
	TCFT3.Mining.OreList = {}
	local num = tonumber(params.numitems)
	if (num>0) then
		for _, orename in ipairs(params.ores) do
			table.insert(TCFT3.Mining.OreList, orename)
		end
	end
	TCFT3.Mining.MineralSearch.LastOreUpdate = os.time()
	TCFT3.Mining.MineralSearch.UpdateOrelist()
end

-- Receving ore-search result - also called mineral search
TCFT3.Events[850] = function(params)
	if (TCFT3.Mining.MineralSearch.Cancelled==false) then	
		ProcessEvent("TCFT3_UPDATEPROGRESSBAR", params.progress)
		local num = tonumber(params.numitems)
		if (num>0) then
			for _, ore in ipairs(params.ores) do
				table.insert(TCFT3.Mining.MineralSearch.Results, ore)
			end
			TCFT3.SendData({ cmd="601" });
		else
			ProcessEvent("TCFT3_PROGRESSBARDONE")
			TCFT3.Mining.MineralSearch.UpdateMineralSearchTable()
		end
	else
		TCFT3.SendData({ cmd="602" })
		TCFT3.Mining.MineralSearch.Results = {}
		TCFT3.Mining.MineralSearch.UpdateMineralSearchTable()
	end
end
	
-- Receiving part of sector data (this can be called repeatly until server buffer is empty)
TCFT3.Events[900] = function(params)
	if (params.numitems>0) then
		local sectorid = GetCurrentSectorid()
		for _, obj in ipairs(params.objects) do
			if (tonumber(obj.sectorid)==sectorid) then
				-- table.insert(TCFT3.Mining.RoidInfo, obj)
				TCFT3.Mining.RoidInfo[tonumber(obj.objectid)] = obj
			end
		end
		TCFT3.SendData({ cmd="701" })
	--else
		--TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.NORMAL .. "Sector Loaded.", true)
	end
end

-- Receiving some system information (repeatly called as server sends chunks)
TCFT3.Events[910] = function(params)
	if (TCFT3.Mining.LoadSystemCancelled==false) then -- Did the user cancel this operation?
		ProcessEvent("TCFT3_UPDATEPROGRESSBAR", params.progress)
		if (params.numitems>0) then
			local systemid = GetCurrentSystemid() - 1
 			local startsec = systemid*256+1
			local endsec = systemid*256+257
			for _, obj in ipairs(params.objects) do
				table.insert(TCFT3.Mining.RoidInfoSystem, obj)
			end
			TCFT3.SendData({ cmd="711" })
		else
			ProcessEvent("TCFT3_PROGRESSBARDONE")
			TCFT3.Mining.LoadedSystem = TCFT3.Mining.CurrentSystem
			TCFT3.Mining.MiningMap.UpdateMapInfo()
		end
	else
		TCFT3.SendData({ cmd="712" })
		TCFT3.Mining.RoidInfoSystem =  {}
	end
end

-- Receive list of stations that needs update
TCFT3.Events[950] = function(params)
	if (TCFT3.Trading.StationUpdatesCancelled==false) then -- Did the user cancel this operation?
		ProcessEvent("TCFT3_UPDATEPROGRESSBAR", params.progress)
		if (params.numitems>0) then
			for _, item in ipairs(params.items) do
				item.location = ShortLocationStr(item.sectorid) or "?"
				item.stationname = GetStationName(item.stationid) or "?"
				if (tonumber(item.mark)==1) then
					item.fgcolor = "255 255 255"
				elseif (tonumber(item.mark)==2) then
					item.fgcolor = "240 0 0"
				else
					item.fgcolor = "66 220 66"
				end
				TCFT3.Trading.StationUpdateList:additem(item)
			end
			TCFT3.SendData({ cmd="351" })
		else
			ProcessEvent("TCFT3_PROGRESSBARDONE")
			TCFT3.Trading.StationUpdateList:update()
		end
	else
		TCFT3.SendData({ cmd="352" })
	end
end

-- Receive some orelist data
TCFT3.Events[970] = function(params)
	if (TCFT3.Mining.LoadOrelistCancelled==false) then
		ProcessEvent("TCFT3_UPDATEPROGRESSBAR", params.progress)
		if (params.numitems>0) then
			for _, object in ipairs(params.objects) do
				object.system = SystemNames[object.systemid] or "?"
				TCFT3.Mining.OrelistPrSystem.OreList:additem(object)
				-- printtable(object)
			end
			TCFT3.SendData({ cmd="731" })
		else
			ProcessEvent("TCFT3_PROGRESSBARDONE")
			TCFT3.Mining.OrelistPrSystem.OreList:update()
		end	
	else
		TCFT3.SendData({ cmd="732" })
	end
end

-- Version - check if client version are current
TCFT3.Events[1000] = function(params)
	TCFT3.SendData({ cmd='400'}) -- Update available trade items
	TCFT3.ServerVersion = params.version
	if (params.version ~= TCFT3.Version) then
		--TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.YELLOW .. "*** There is a new version of the client available (version " .. params.version .. "). Check changelog and download at http://vo.aoczone.net", true)
	else
		--TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.NORMAL .. "*** Client version is current.")
	end
	TCFT3.SendData({ cmd='450'}) -- Update station list
	TCFT3.SendData({ cmd='610'}) -- Update station list
end

-- Message about stations that need attention
TCFT3.Events[1100] = function(params)
	--TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.RED .. "*** There are stations where trade data are out of date. Check \"Stations\" under Trade Tab in TCFT3.", true)
end

-- Receive navigation data
TCFT3.Events[1200] = function(params)
	TCFT3.PloziNav.Routes = unspickle(params.navdata)
	TCFT3.PloziNav.Editor.LoadData()
	TCFT3.PloziNav.Editor.LoadSystemRoutes()
	local broadcast = true
	if (TCFT3.PloziNav.UI.EditDialog.visible=="YES") then
		broadcast = false
		--TCFT3.MessageDialog("Navigation Computer", "Data successfully fetched from the server.")
	end
	--TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.NORMAL .. "Navigation data successfully fetched from server.", broadcast)
end

-- Data saved ok
TCFT3.Events[1211] = function(params)
	TCFT3.MessageDialog("Navigation Editor", "There was an error saveing data to the server.")
end

-- Error saving data
TCFT3.Events[1212] = function(params)
	TCFT3.ErrorDialog("Navigation Editor", "There was an error saveing data to the server.")
end

-- No access for saving data
TCFT3.Events[1213] = function(params)
	TCFT3.ErrorDialog("Navigation Editor", "There was an error saveing data to the server.")
end

-- Response from last-seen
TCFT3.Events[1260] = function(params)
	console_print("1260 " .. params.numitems)
	TCFT3.LastSeen.List:clear()
	if (tonumber(params.numitems)>0) then
		for _, player in ipairs(params.seenlist) do
			local guildtag = ""
			if (player.guildtag~=nil) and (player.guildtag~="") then
				guildtag = "[" .. player.guildtag .. "]"
			end
			local rec = { name=player.name, date=os.date("%Y-%m-%d %H:%M", player.ts), location=ShortLocationStr(player.sectorid), guildtag=guildtag, shipname=player.shipname, health=player.health }
			TCFT3.LastSeen.List:additem(rec)
		end
		TCFT3.LastSeen.List:update()
	else
		TCFT3.LastSeen.List.NUMLIN=2
		TCFT3.LastSeen.List["2:1"] = "Nothing found."
	end
end

-- Live feed for lastseen
TCFT3.Events[1265] = function(params)
	-- HUD:PrintSecondaryMissionMsg(msg)
	-- HUD:PrintSecondaryMsg(msg)
	local players = params.players or {}
	local msgstr = ""
	for idx, player in ipairs(players) do
		if (player.name~=GetPlayerName()) and (tonumber(player.sectorid)~=GetCurrentSectorid()) then -- NOT Player itself or same sector
			local guild = "" 
			if ((player.guildtag or "")~="") then
				guild = "[" .. player.guildtag .. "] "
			end
			local factioncolor = ""
			if ((tonumber(player.faction) or 0) > 0) then
				factioncolor = rgbtohex(FactionColor_RGB[tonumber(player.faction)])
			else
				factioncolor = TCFT3.colors.RED
			end
			local location = "Unknown Location"
			local sectid = player.sectorid or -1
			if (sectid>0) then
				location = ShortLocationStr(sectid)
			end
			local shipname = (player.shipname) or ""
			if (shipname~="") then shipname = " (Flying " .. shipname .. ")" end
			local txt = factioncolor .. guild .. player.name .. TCFT3.colors.RED .. " spotted in " .. location .. shipname .. TCFT3.colors.NORMAL
			-- Check if this notification already received - if so then dont add again
			local exists = false
			for _, noti in ipairs(TCFT3.LastSeen.Notifications) do
				if (noti.txt==txt) then
					exists=true
					break
				end
			end		
			if (exists==false) then
				table.insert(TCFT3.LastSeen.Notifications, { time=os.time(), txt=txt, label=nil })
			end
		end -- Player itself or same sector
	end
end

local function updateLastSeenBars(res, barnum)
	local max=0
	for fac, num in pairs(res) do
		num = tonumber(num)
		fac = tonumber(fac)
		if (fac>0) and (fac<4) then
			TCFT3.Players.LastSeenBars[barnum][fac]:setvalue(num)
			TCFT3.Players.LastSeenBars[barnum][fac]:settitle(FactionName[fac] .. " " .. num)
			if (num>max) then max = num end
		end
	end
	for fac, num in pairs(res) do
		fac = tonumber(fac)
		if (fac>0) and (fac<4) then
			TCFT3.Players.LastSeenBars[barnum][fac]:setminmax(0, max)
		end
	end
end

-- Last Seen statistics
TCFT3.Events[1300] = function(params)
	updateLastSeenBars(params.spots.overall, 5)
	updateLastSeenBars(params.spots.itani, 1)
	updateLastSeenBars(params.spots.serco, 2)
	updateLastSeenBars(params.spots.uit, 3)
	updateLastSeenBars(params.spots.grey, 4)
end


-- Receive list of guild members
TCFT3.Events[1500] = function(params)
	TCFT3.Guild.MemberList:clear()
	TCFT3.GuildMembers = {}
	local num = tonumber(params.numitems) or 0
	if (num>0) then
		local members = params.members or {}
		if (#members>0) then
			for _, member in ipairs(members) do
				local trank = tonumber(member.rank)
				member.rank = TCFT3.Guild.MemberRanks[tonumber(member.rank)]
				-- 128 0 128
				-- 153 51 102
				-- 255 255 0
				if (trank==0) then -- Probie
					member.fgcolor = "55 185 53"
				elseif (trank==1) then -- Member
					member.fgcolor = "220 145 23"
				elseif (trank==2) then -- Lieutenant
					member.fgcolor = "180 180 180"
				elseif (trank==3) then -- Council/Lt.
					member.fgcolor = "180 180 180"
				elseif (trank==4) then -- Council
					member.fgcolor = "255 185 50"
				elseif (trank==5) then -- Commander
					member.fgcolor = "255 255 255"
				elseif (trank==6) then -- Applicant
					member.fgcolor = "255 20 20"
				else -- Unknown
					member.fgcolor = "15 15 15"
				end
				TCFT3.Guild.MemberList:additem(member)
				TCFT3.GuildMembers[string.lower(member.name)] = tonumber(member.rank) or 0
			end
			TCFT3.Guild.MemberList:update()
		else
			TCFT3.Guild.MemberList.NUMLIN=2
			TCFT3.Guild.MemberList:setcell(2, 2, "No members found.")
		end
	else
		TCFT3.Guild.MemberList.NUMLIN=2
		TCFT3.Guild.MemberList:setcell(2, 2, "No members found.")
	end
end


