--[[
TCFT3
(c) By Plozi
Version 1.0.0

** Changelog

]]

TCFT3.LastSeen = TCFT3.LastSeen or {}
TCFT3.LastSeen.Timer = Timer()
TCFT3.LastSeen.NotifyTimer = Timer()
TCFT3.LastSeen.Timeout = 10 -- 15 seconds
TCFT3.LastSeen.Notifications = {}
TCFT3.LastSeen.NotifyTimeout = 15 -- 8 seconds

dofile("ui/ui_lastseen.lua")

TCFT3.LastSeen.PlayerListCheck = {} -- List of unknown players to check for

TCFT3.DisplayLastSeenList = function()
	if (TCFT3.LastSeen.MainWindow==nil) then
		TCFT3.LastSeen.CreateMainWindow()
	end
	PopupDialog(TCFT3.LastSeen.MainWindow,iup.CENTER, iup.CENTER)
	-- TCFT3.LastSeen.Update()
end

TCFT3.LastSeen.Update = function()
	TCFT3.LastSeen.List:clear()
	TCFT3.LastSeen.List.NUMLIN=2
	if (TCFT3.Connection.isLoggedin~=true) then
		TCFT3.LastSeen.List["2:2"] = "Not logged in..."
	else
		TCFT3.LastSeen.List["2:2"] = "Updating, please wait..."
		local name=""
		local guildtag=""
		local sectorid=GetCurrentSectorid() 
		local typeinput = tonumber(TCFT3.LastSeen.Type.value)
		if (typeinput==1) then -- Current sector
			-- Nothing to do actually
			sectorid = GetCurrentSectorid()
		elseif (typeinput==2) then -- Player name
			name=TCFT3.LastSeen.InputData.value
			if (name=="") then
				TCFT3.MessageDialog("LastSeen", "No Player name given. Try again.")
				return
			end
		elseif (typeinput==3) then -- Guild
			guildtag=TCFT3.LastSeen.InputData.value
			if (guildtag=="") then
				TCFT3.MessageDialog("LastSeen", "No guild-tag given. Try again.")
				return
			end
		elseif (typeinput==4) then
			sectorid = -1
		end
		
		TCFT3.SendData({ cmd="1260", sectorid=sectorid, name=name, guildtag=guildtag })
	end
end

-- Check to see if we have any notification to send
TCFT3.LastSeen.CheckSendData = function()
	local tmplist = {}
	if (#TCFT3.LastSeen.PlayerListCheck>0) then -- Check if we have any unknowns
		for idx, charid in ipairs(TCFT3.LastSeen.PlayerListCheck) do
			if (charid>0) then
				local name = GetPlayerName(charid)
				if (name==nil) then -- no name - we just presume he left sector
					TCFT3.LastSeen.PlayerListCheck[idx] = -1
				else
					if (string.find(name, "reading transponder")==nil) then -- not unknown anymre...
						if (string.sub(name, 1, 1)=="*") then -- No NPC's
							TCFT3.LastSeen.PlayerListCheck[idx] = -1
						else
							if (GetPlayerName()==name) then -- Not our self
								TCFT3.LastSeen.PlayerListCheck[idx] = -1
							else
								local faction = GetPlayerFaction(charid) or 0
								local guildtag = GetGuildTag(charid) or ""
								local health = GetPlayerHealth(charid) or 0.0
								local shipname = GetPrimaryShipNameOfPlayer(charid) or ""
								table.insert(tmplist, { name=name, sectorid=GetCurrentSectorid(), guildtag=guildtag, faction=faction, health=health, shipname=shipname, charid=charid })
								TCFT3.LastSeen.PlayerListCheck[idx] = -1
							end
						end
					end
				end
			else
				TCFT3.LastSeen.PlayerListCheck[idx] = -1
			end
		end
		local tmp = {}
		for idx, charid in ipairs(TCFT3.LastSeen.PlayerListCheck) do
			if (charid~=-1) then
				table.insert(tmp, charid)
			end
		end
		TCFT3.LastSeen.PlayerListCheck = tmp
	end
	if (#tmplist>0) then
		TCFT3.SendData({ cmd="1250", players=tmplist })
	end
	TCFT3.LastSeen.Timer:SetTimeout(TCFT3.LastSeen.Timeout*1000, TCFT3.LastSeen.CheckSendData)
end

-- Check if we have any notifications
TCFT3.LastSeen.CheckNotify = function()
	local now = os.time()
	for idx, n in ipairs(TCFT3.LastSeen.Notifications) do
		if ((TCFT3.Settings.Data.chk_autonotifylastseen or "ON")=="ON") then
			if (n.label==nil) then
				n.label = iup.label{title=n.txt, font=Font.H3*HUD_SCALE, alignment="ACENTER", expand="HORIZONTAL"}
				iup.Append(TCFT3.LastSeen.NotifyBox, n.label)
				TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. n.txt, true)
			end
		end
		if (now - n.time > TCFT3.LastSeen.NotifyTimeout) then
			if (n.label~=nil) then
				iup.Detach(n.label)
				iup.Destroy(n.label)
			end
			n.txt = nil
		end
	end
	local tmp = {}
	for _, n in ipairs(TCFT3.LastSeen.Notifications) do
		if (n.txt~=nil) then table.insert(tmp, n) end
	end
	TCFT3.LastSeen.Notifications = tmp
	if (TCFT3.LastSeen.NotifyBox2~=nil) then
		TCFT3.LastSeen.NotifyBox2.visible = (#TCFT3.LastSeen.Notifications>0 and "YES") or "NO"
	end
	TCFT3.LastSeen.NotifyTimer:SetTimeout(1000, TCFT3.LastSeen.CheckNotify)
end

-- Event
TCFT3.LastSeen.EventPlayerEnteredSector = function(_, charid)
	table.insert(TCFT3.LastSeen.PlayerListCheck, charid)
end

-- Check to see if anybody in the bar when you dock
TCFT3.LastSeen.EventPlayerEnteredStation = function(_, params)
	if (PlayerInStation()) then
		local inbar = GetBarPatrons()
		for idx, charid in ipairs(inbar) do
			table.insert(TCFT3.LastSeen.PlayerListCheck, charid)
		end
	end
end

-- Event
TCFT3.LastSeen.EventHudShow = function()
	-- TCFT3.LastSeen.Notify = TCFT3.LastSeen.Notify or iup.label{title="TEST", font=Font.H4*HUD_SCALE, alignment="ACENTER", expand="HORIZONTAL"}
	if (TCFT3.LastSeen.NotifyBox==nil) then
		TCFT3.LastSeen.NotifyBox = iup.vbox { margin="8x4", expand="YES", alignment="ACENTER" }
		TCFT3.LastSeen.NotifyBox2 = iup.vbox {
			visible="NO",
			iup.fill { size="%69" },
			iup.hbox {
				iup.fill {},
				iup.hudrightframe {
					border="2 2 2 2",
					iup.vbox {
						expand="NO",
						iup.hudrightframe {
							border="0 0 0 0",
							iup.hbox {
								TCFT3.LastSeen.NotifyBox,
							},
						},
					},
				},
				iup.fill {},
			},
			alignment="ACENTER",
			gap=2
		}
		if ((TCFT3.Settings.Data.chk_autonotifyhud or "ON")=="ON") then iup.Append(HUD.pluginlayer, TCFT3.LastSeen.NotifyBox2) end
		TCFT3.LastSeen.NotifyTimer:SetTimeout(1000, TCFT3.LastSeen.CheckNotify)
	end
end

TCFT3.LastSeen.EventSectorChanged = function(_, data)
	-- Add all players in sector to lastseen
	ForEachPlayer(
		function (charid)
			if (charid~=0) then table.insert(TCFT3.LastSeen.PlayerListCheck, charid) end
		end
	)
end

TCFT3.LastSeen.Timer:SetTimeout(TCFT3.LastSeen.Timeout*1000, TCFT3.LastSeen.CheckSendData)

RegisterEvent(TCFT3.LastSeen.EventPlayerEnteredSector, "PLAYER_ENTERED_SECTOR")
RegisterEvent(TCFT3.LastSeen.EventPlayerEnteredStation, "ENTERED_STATION")
RegisterEvent(TCFT3.LastSeen.EventSectorChanged, "SECTOR_CHANGED")
RegisterEvent(TCFT3.LastSeen.EventHudShow, "HUD_SHOW")
