--[[
PloziNav
(c) By Plozi
Version 2.0.0

** Changelog

2.0.5 - Added auto pilot.  
2.0.0 - PloziNav Data store on server and editable directly from the plugin
1.0.5 - Now also roundtrips should be plotted correctly - good for proc missions



]]

-- Navigation computer
TCFT3.PloziNav = TCFT3.PloziNav or {}

-- Navigation Computer Editor
dofile('navigation_editor.lua')
-- Load UI definitions
dofile('ui/ui_navigation.lua')
-- Load the navigation data file
-- dofile('PloziNav.Data.lua')
TCFT3.PloziNav.Routes = {}
TCFT3.PloziNav.LastRoute = {}
TCFT3.PloziNav.DestinationSector = 0
TCFT3.PloziNav.UserInfo = {}

-- Add controls to nav screen
TCFT3.PloziNav.AddControls = function(tab)
	local panel = TCFT3.PloziNav.UI.CreatePanel()
	local container = iup.GetBrother(iup.GetBrother(iup.GetNextChild(tab)))
	container:append(
		iup.pdasubframe_nomargin{
			iup.hbox {
				iup.vbox {
					iup.hbox {
						iup.fill {},
						iup.label { title="Plozi's Navigation Computer" },
						iup.fill {},
					},
					iup.hbox {
						iup.fill {},
						panel,
						iup.fill {},
						margin="0x5",
					},
					margin="5x5",
				},
			},
			expand="HORIZONTAL",
		}
	)
end

TCFT3.PloziNav.GetRoute = function(from, to)
	local res = {}
	if TCFT3.PloziNav.Routes[from.."-"..to] then
		res = TCFT3.PloziNav.Routes[from.."-"..to]
	end
	return res
end

-- SplitSectorID - return SystemID, X, Y

-- PlotRoute
TCFT3.PloziNav.PlotRoute = function()
	local currSectorId = GetCurrentSectorid()
	local destSectorId = NavRoute.GetFinalDestination()

	local currRoute = NavRoute.GetCurrentRoute()
	local currFullRoute = GetFullPath(currSectorId, currRoute)

	if destSectorId==nil then
		TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.RED .. "Unknown destination sector.", true)
		return
	end
	if destSectorId==currSectorId and #currFullRoute<1 then
		TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.RED .. "Start and destination sector are the same - nothing to plot.", true)
		return
	end

	local fullRoute = {}
	local newRoute = {}
	local done = {}
	local prev = 0
	for i,v in ipairs(currFullRoute) do
		if prev ~= v then
			table.insert(fullRoute, v)
			prev = v
		end
	end
	local to = ""
	local from = ""
	for i=1,#fullRoute-1 do
		from = fullRoute[i]
		to = fullRoute[i+1]
		if from~=nil and to~=nil then
			local points = TCFT3.PloziNav.GetRoute(from, to)
			if (#points) then
				for i,v in ipairs(points) do
					table.insert(newRoute, tonumber(v))		
				end
			else
				table.insert(newRoute, from)
			end
		end
	end
	table.insert(newRoute, to) -- Be sure to add the wery last location

	if #newRoute>0 then
		NavRoute.clear()		
		-- NavRoute.SetFinalDestination(destSectorId)
		NavRoute.SetFullRoute(newRoute)
		TCFT3.PloziNav.LastRoute = {}
		for _, hop in ipairs(newRoute) do -- Save route
			table.insert(TCFT3.PloziNav.LastRoute, hop)
		end
		TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.NORMAL .. "New route plotted (" .. ShortLocationStr(currSectorId) .. " -> " .. ShortLocationStr(destSectorId) .. ")", true)
	else
		TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.RED .. "Could not plot route. Unknown error.", true)
		return
	end
end

-- Repeat last route
TCFT3.PloziNav.RepeatLast = function()
	NavRoute.clear()
	NavRoute.SetFullRoute(TCFT3.PloziNav.LastRoute)
end


TCFT3.PloziNav.LoadServerNavData = function()
	local broadcast = (TCFT3.PloziNav.UI.EditDialog.visible=="YES" and false) or true
	--TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.NORMAL .. "Attempting to load navigation data from server...", broadcast)
	TCFT3.SendData({ cmd="1200" })
end

-- Initialize PloziNav
TCFT3.PloziNav.Initialize = function()
	TCFT3.PloziNav.Editor.Initialize()
	--TCFT3.AddStatus(TCFT3.colors.GREEN2 .. "(TCFT3) " .. TCFT3.colors.NORMAL .. "Navigation computer initialized.", true)
	TCFT3.PloziNav.LoadServerNavData()
end

--[[
Route editing functions
]]

-- Initialize - add controls to the nav-tabs
TCFT3.PloziNav.AddControls(PDAShipNavigationTab)
TCFT3.PloziNav.AddControls(StationPDAShipNavigationTab)
TCFT3.PloziNav.AddControls(CapShipPDAShipNavigationTab)

