--[[

Navigation computer route editor

]]


TCFT3.PloziNav.Editor = TCFT3.PloziNav.Editor or {}
TCFT3.PloziNav.Editor.Data = nil
TCFT3.PloziNav.Editor.EditMode = false
TCFT3.PloziNav.Editor.EditPath = {}
TCFT3.PloziNav.Editor.CurrentSystem = 1
TCFT3.PloziNav.Editor.CurrentSector = 1

TCFT3.PloziNav.Editor.MapMode = 0 -- 0 is u	niversemap 1 is sector map

-- Display editor
TCFT3.PloziNav.Editor.ShowEditor = function()
	
	if (TCFT3.PloziNav.Editor.Data == nil) then
		TCFT3.PloziNav.Editor.LoadData()
	end
	TCFT3.PloziNav.UI.EditDialog:popup(iup.CENTER, iup.CENTER)
end

-- Load map
TCFT3.PloziNav.Editor.LoadMap = function()
	local mappath, clickid
	if (TCFT3.PloziNav.Editor.MapMode == 0) then
		--TCFT3.PloziNav.UI.EditDialog:map()
		TCFT3.PloziNav.UI.navmap:loadmap(1, "lua/maps/universemap.lua", 0)
		clickid = TCFT3.PloziNav.Editor.CurrentSystem - 1
	else
		mappath = string.format("lua/maps/system%02dmap.lua", TCFT3.PloziNav.Editor.CurrentSystem)
		--TCFT3.PloziNav.UI.EditDialog:map()
		TCFT3.PloziNav.UI.navmap:loadmap(2, mappath, TCFT3.PloziNav.Editor.CurrentSystem - 1)
		clickid = TCFT3.PloziNav.Editor.CurrentSector
		-- TCFT3.PloziNav.UI.navmap
	end
	TCFT3.PloziNav.UI.navmap.clickedid = clickid
	TCFT3.PloziNav.UI.navmap.currentid = GetCurrentSectorid()
end

-- Zoom button clicked
TCFT3.PloziNav.Editor.Zoom = function(btn)
	if (TCFT3.PloziNav.Editor.MapMode==1) then
		TCFT3.PloziNav.Editor.MapMode = 0
		btn.title = "Zoom to System"
		TCFT3.PloziNav.UI.Savebtn.active="NO"
		TCFT3.PloziNav.Editor.EditPath = {}
		-- TCFT3.PloziNav.UI.navmap.zoom = 1
	else
		TCFT3.PloziNav.Editor.MapMode = 1
		btn.title = "Zoom to Universe"
		-- TCFT3.PloziNav.UI.navmap.zoom = 10
	end
	TCFT3.PloziNav.Editor.LoadMap()
end

-- Save Route to db...
TCFT3.PloziNav.Editor.SaveRouteToDb = function()
	local from = TCFT3.PloziNav.Editor.EditPath[1]
	local to = TCFT3.PloziNav.Editor.EditPath[#TCFT3.PloziNav.Editor.EditPath]
	local newRoute = {}
	for i,val in ipairs(TCFT3.PloziNav.Editor.EditPath) do
		if (i>1) then
			table.insert(newRoute, val)
		end
	end
	TCFT3.PloziNav.Routes[from.."-"..to] = newRoute
	TCFT3.PloziNav.Editor.LoadData()
	TCFT3.PloziNav.Editor.LoadSystemRoutes()
end

-- Save Route button pressed
TCFT3.PloziNav.Editor.SaveRoute = function(btn)
	local from = TCFT3.PloziNav.Editor.EditPath[1]
	local to = TCFT3.PloziNav.Editor.EditPath[#TCFT3.PloziNav.Editor.EditPath]
	if (TCFT3.PloziNav.Routes[from.."-"..to] ~= nil) then
		TCFT3.ConfirmDialog("Confirm save", 
			"Route already exists - replace?",
			"Yes",
			TCFT3.PloziNav.Editor.SaveRouteToDb,
			"No",
			nil
		)
	else
		TCFT3.PloziNav.Editor.SaveRouteToDb()
	end
end

-- Mouse over a sector or system
TCFT3.PloziNav.Editor.Map_Mouseover = function(index, str)
	local note = ""
	local id = tonumber(index) or index
	local system
	local conqueringfaction
	local desc = ""
	local system_notes_heading = '\n[Sector Notes]\n'

	if (TCFT3.PloziNav.Editor.MapMode == 0) then
		if SystemNotes[id+1] and #SystemNotes[id+1].name > 0  then -- system
			-- need to format for system text, not \n terminated
			note = '\n'..system_notes_heading..(SystemNotes[id+1].name or "")
		end
	elseif (TCFT3.PloziNav.Editor.MapMode == 1) then
		system = GetSystemID(id)
		if SystemNotes[system] and SystemNotes[system][id] then-- sector
			note = system_notes_heading..(SystemNotes[system][id]:gsub("\\n","\n") or "")
		end
		note = (GetBotSightedInfoForSector(id) or "")..note
		conqueringfaction = GetConqueredStatus(id)
	end
	if conqueringfaction then
		desc = "Currently controlled by "..FactionArticle[conqueringfaction].." "..FactionNameFull[conqueringfaction].."\n\n"..(str and string.gsub(str, "|", "\n") or "")..note
	else
		desc = (str and string.gsub(str, "|", "\n") or "")..note
	end
	TCFT3.PloziNav.Editor.SectorInfo.value = desc
	-- desc.index = index
end

TCFT3.PloziNav.Editor.Map_Click = function(index, modifiers)
	if (TCFT3.PloziNav.Editor.MapMode == 0) then
		TCFT3.PloziNav.Editor.CurrentSystem = index + 1
		TCFT3.PloziNav.Editor.LoadSystemRoutes()
	else 
		TCFT3.PloziNav.Editor.CurrentSector = index
		if string.byte(modifiers, 1) == iup.K_S then
			-- Shift-Click
			table.insert(TCFT3.PloziNav.Editor.EditPath, index)
			TCFT3.PloziNav.UI.Savebtn.active = "YES"
		else
			-- Normal click
			TCFT3.PloziNav.Editor.EditPath = { index }
			TCFT3.PloziNav.UI.Savebtn.active = "NO"
		end
		TCFT3.PloziNav.UI.navmap:setpath(GetFullPath(TCFT3.PloziNav.Editor.EditPath[1], TCFT3.PloziNav.Editor.EditPath))
	end
end

-- Load current navigation data
TCFT3.PloziNav.Editor.LoadData = function()
	TCFT3.PloziNav.Editor.Data = {}
	for system,system_data in pairs(TCFT3.PloziNav.Routes) do
		local pos = string.find(system, '-', 1)
		local from = tonumber(string.sub(system, 1, pos-1)) or -1
		local to = tonumber(string.sub(system, pos+1)) or -1
		local sysid = GetSystemID(from)
		local data = {}
		data.desc = ShortLocationStr(from) .. " -> " .. ShortLocationStr(to)
		data.from = from
		data.to = to
		data.data = system_data
		TCFT3.PloziNav.Editor.Data[sysid] = TCFT3.PloziNav.Editor.Data[sysid] or {}
		table.insert(TCFT3.PloziNav.Editor.Data[sysid], data)
 	end
	local cmp = function(a,b)
		local ax1 = AbbrLocationStr(a.from)
		local ax2 = AbbrLocationStr(a.to)
		local bx1 = AbbrLocationStr(b.from)
		local bx2 = AbbrLocationStr(b.to)
		if (ax1==bx1) then
			return ax2 < bx2
		end
		return ax1 < bx1
	end
	for sid,sysdata in pairs(TCFT3.PloziNav.Editor.Data) do
		table.sort(sysdata, cmp)
	end
end

-- Load routes for selected system - TCFT3.PloziNav.
TCFT3.PloziNav.Editor.LoadSystemRoutes = function()
	TCFT3.PloziNav.UI.RouteList[1] = nil
	local num = 1
	if (TCFT3.PloziNav.Editor.Data[TCFT3.PloziNav.Editor.CurrentSystem] ~= nil) then
		for _,sysdata in ipairs(TCFT3.PloziNav.Editor.Data[TCFT3.PloziNav.Editor.CurrentSystem]) do
			TCFT3.PloziNav.UI.RouteList[num] = sysdata.desc
			num = num + 1
		end
	else
		TCFT3.PloziNav.UI.RouteList[num] = "No routes"
	end
end

-- Click on a route - set in on the navmap
TCFT3.PloziNav.Editor.RouteClick = function(list, text, index, selection)
	local record
	if (TCFT3.PloziNav.Editor.Data[TCFT3.PloziNav.Editor.CurrentSystem] ~= nil) then
		if (TCFT3.PloziNav.Editor.Data[TCFT3.PloziNav.Editor.CurrentSystem][index] ~= nil) then
			record = TCFT3.PloziNav.Editor.Data[TCFT3.PloziNav.Editor.CurrentSystem][index]
			TCFT3.PloziNav.UI.navmap:setpath(GetFullPath(record.from, record.data))
		end
	end
end

local function print_r(tbl)
	for key,val in pairs(tbl) do
		if (type(val)=='table') then
			PloziTools.msg("*** " .. key)
			print_r(val)
		else
			if (type(val)~='function') then
				PloziTools.msg(key .. " = " .. val)
			end
		end
	end
end

-- Refresh data from server
TCFT3.PloziNav.Editor.RefreshNavData2 = function()
	TCFT3.PloziNav.LoadServerNavData()
end

TCFT3.PloziNav.Editor.RefreshNavData = function()
	TCFT3.ConfirmDialog("Confirm Refresh", "This will replace all your local changes\nwith data from the server.\nContinue refreshing from server?", "YES", TCFT3.PloziNav.Editor.RefreshNavData2, "NO", nil)
end

TCFT3.PloziNav.Editor.SendDataToServer2 = function()
	TCFT3.SendData({ cmd="1210", data=spickle(TCFT3.PloziNav.Routes) })
end

TCFT3.PloziNav.Editor.SendDataToServer = function(self)
	TCFT3.ConfirmDialog("Send Data To Server", 
		"This will replace the navigation data on the server\nwith your local data.\nContinue?",
		"Yes", TCFT3.PloziNav.Editor.SendDataToServer2,
		"No", nil		
	)
end

-- Delete selected Route
TCFT3.PloziNav.Editor.DeleteRoute2 = function()
	local tmpRoutes = {}
	for sys,data in pairs(TCFT3.PloziNav.Routes) do
		if (sys ~= TCFT3.PloziNav.Editor.SystemToDelete) then
			tmpRoutes[sys] = data
		end
	end
	TCFT3.PloziNav.Routes = tmpRoutes
	TCFT3.PloziNav.Editor.LoadData()
	TCFT3.PloziNav.Editor.LoadSystemRoutes()
end

TCFT3.PloziNav.Editor.DeleteRoute = function(self)
	local idx = tonumber(TCFT3.PloziNav.UI.RouteList.value)
	if (idx == 0) then
		TCFT3.MessageDialog("Delete route", "You have to select a route to delete.")
	else
		if (TCFT3.PloziNav.Editor.Data[TCFT3.PloziNav.Editor.CurrentSystem] ~= nil) then
			if (TCFT3.PloziNav.Editor.Data[TCFT3.PloziNav.Editor.CurrentSystem][idx] ~= nil) then
				local delrec = TCFT3.PloziNav.Editor.Data[TCFT3.PloziNav.Editor.CurrentSystem][idx]
				if (delrec) then
					local pos = delrec.from .. '-' .. delrec.to
					local delSystems = ShortLocationStr(delrec.from) .. " -> " .. ShortLocationStr(delrec.to)
					TCFT3.PloziNav.Editor.SystemToDelete = pos
					TCFT3.ConfirmDialog("Confirm Delete", "Delete route:\n" .. delSystems, "YES", TCFT3.PloziNav.Editor.DeleteRoute2, "NO", nil)
				end
			end
		end

	end
end

TCFT3.PloziNav.Editor.Initialize = function()
	TCFT3.PloziNav.Editor.CurrentSystem = GetCurrentSystemid() or 1
	TCFT3.PloziNav.Editor.CurrentSector = GetCurrentSectorid() or 1
end

