-- TCFT3 UI
-- Trade Tab

TCFT3.Tools = TCFT3.Tools or {}

-- ITEM LOOKUP
local function createItemlookupTab()
	local container

	TCFT3.Trading.LookupList = TCFT3.MatrixList:new()
	TCFT3.Trading.LookupList:addcolumn(TCFT3.MatrixList.column.new("itemname", "Item Name", "ALEFT", { "S:A:itemname" }, 20))
	TCFT3.Trading.LookupList:addcolumn(TCFT3.MatrixList.column.new("sellprice", "Station Sell Price", "ARIGHT", { "N:A:sellprice" }, 9, { postfix=" c" }))
	TCFT3.Trading.LookupList:addcolumn(TCFT3.MatrixList.column.new("buyprice", "Station Buy Price", "ARIGHT", { "N:D:buyprice" }, 9, { postfix=" c" }))
	TCFT3.Trading.LookupList:addcolumn(TCFT3.MatrixList.column.new("xp", "XP", "ARIGHT", { "N:D:xp" }, 6, { replace={ ["0"]="--" }, postfix=" xp" }))
	TCFT3.Trading.LookupList:addcolumn(TCFT3.MatrixList.column.new("location", "Location", "ALEFT", { "S:A:location" }, 15))
	TCFT3.Trading.LookupList:addcolumn(TCFT3.MatrixList.column.new("station", "Station", "ALEFT", { "S:A:station" }, 15))
	TCFT3.Trading.LookupList:addcolumn(TCFT3.MatrixList.column.new("context", "Context", "ALEFT", { "S:A:context" }, 15))
	TCFT3.Trading.LookupList:addcolumn(TCFT3.MatrixList.column.new("dateupdated", "Last Updated", "ARIGHT", { "S:D:dateupdated" }, 11))
	TCFT3.Trading.LookupList:init()

	TCFT3.Trading.LookupItemType = iup.stationsublist { "Anything", "Commodities", "Ships", "Weapons", "Batteries"; dropdown='yes', size='%10x', expand='NO', visible_items=15 }
	TCFT3.Trading.LookupItemType.action = TCFT3.Trading.LookupItemTypeChanged
	TCFT3.Trading.LookupItemList = iup.stationsublist { dropdown='yes', size='%20x', expand='NO', visible_items=15 } -- visible_items=25 to 15 -- tokkana
	TCFT3.Trading.LookupBoughtSold = iup.stationsublist { "Bought and Sold", "Bought", "Sold", "Wanted"; dropdown='yes', size='%10x', expand='NO', visible_items=5 }
	TCFT3.Trading.LookupSystem = iup.stationsublist { "Any system"; dropdown="YES", size="%15x", expand="NO", visible_items=15 } -- visible_items=25 to 15 -- tokkana
	TCFT3.Trading.TradeLookupButton = iup.stationbutton {
		title="Lookup",
		action = function(self)
			TCFT3.Trading.DoTradeLookup()
		end,
		active="YES"
	}

	container = iup.vbox {
		margin="4x4",
		iup.hbox {
			iup.label { title="Item lookup", font=Font.H3 }, iup.fill {},
		},
		iup.fill { size="5" },
		iup.hbox {
			expand="YES",
			TCFT3.Trading.LookupItemType,
			TCFT3.Trading.LookupItemList,
			TCFT3.Trading.LookupBoughtSold,
			iup.label { title=" in "},
			TCFT3.Trading.LookupSystem,
			TCFT3.Trading.TradeLookupButton,
			gap="5",
		},
		iup.fill { size=10 },
		iup.hbox {
			expand="YES",
			TCFT3.Trading.LookupList,
		},
	}

	container.tabtitle = "Item lookup"
	container.hotkey = iup.K_l
	function container:OnHelp()
	   TCFT3.ShowHelp("TRADING_ITEMLOOKUP")
	end
	function container:OnActivate(newtab, oldtab)
		if (TCFT3.Connection.isLoggedin==true) then
			TCFT3.SendData({ cmd="400" }) -- Update available trade items
		end
	end
	return container
end

-- STATION COMPARE
local function createStationCompareTab()
	local container

	-- Item | Buy At | Sell At | Profit | Volume | BuyPrice | SellPrice | ItemType | Mass | XP
	TCFT3.Trading.StationCompareList = TCFT3.MatrixList:new()
	TCFT3.Trading.StationCompareList:addcolumn(TCFT3.MatrixList.column.new("itemname", "Item Name", "ALEFT", { "S:A:itemname" }, 22))

	TCFT3.Trading.StationCompareList:addcolumn(TCFT3.MatrixList.column.new("buylocation", "Buy at", "ALEFT", { "S:A:buylocation", "S:A:selllocation" }, 15))
	TCFT3.Trading.StationCompareList:addcolumn(TCFT3.MatrixList.column.new("selllocation", "Sell at", "ALEFT", { "S:A:selllocation", "S:A:buylocation" }, 15))
	TCFT3.Trading.StationCompareList:addcolumn(TCFT3.MatrixList.column.new("cuprofit", "Profit/CU", "ARIGHT", { "N:D:cuprofit" }, 7, { postfix=" c" }))

	TCFT3.Trading.StationCompareList:addcolumn(TCFT3.MatrixList.column.new("volume", "Volume", "ARIGHT", { "N:A:volume" }, 6, { postfix=" cu" }))


	TCFT3.Trading.StationCompareList:addcolumn(TCFT3.MatrixList.column.new("sellprice", "Buy Price", "ARIGHT", { "N:A:sellprice" }, 7, { postfix=" c" })) -- Station sell price - OUR buy price
	TCFT3.Trading.StationCompareList:addcolumn(TCFT3.MatrixList.column.new("buyprice", "Sell Price", "ARIGHT", { "N:D:buyprice" }, 7, { postfix=" c" })) -- Station buy price - OUR sell price

	TCFT3.Trading.StationCompareList:addcolumn(TCFT3.MatrixList.column.new("type", "Item Type", "ALEFT", { "S:A:type" }, 8))
	TCFT3.Trading.StationCompareList:addcolumn(TCFT3.MatrixList.column.new("mass", "Mass", "ARIGHT", { "N:A:mass" }, 7, { postfix=" Kg" }))
	TCFT3.Trading.StationCompareList:addcolumn(TCFT3.MatrixList.column.new("xp", "XP", "ARIGHT", { "N:D:xp" }, 6, { replace={ ["0"]="--" }, postfix = " xp" }))


	TCFT3.Trading.StationCompareList:init()

	TCFT3.Trading.StationCompareStation1 = iup.stationsublist { dropdown='yes', size='%17x', expand='NO', visible_items=15, sort="YES" } -- visible_items=25 to 15 -- tokkana
	TCFT3.Trading.StationCompareStation2 = iup.stationsublist { dropdown='yes', size='%17x', expand='NO', visible_items=15, sort="YES" } -- visible_items=25 to 15 -- tokkana

	TCFT3.Trading.CompareButton = iup.stationbutton {
		title="Compare",
		action = function(self)
			TCFT3.Trading.DoStationCompare()
		end,
		active="YES"
	}
	TCFT3.Trading.SwapAndCompareButton = iup.stationbutton {
		title="Swap And Compare",
		action = function(self)
			local tmp = TCFT3.Trading.StationCompareStation1.value
			TCFT3.Trading.StationCompareStation1.value = TCFT3.Trading.StationCompareStation2.value
			TCFT3.Trading.StationCompareStation2.value = tmp
			TCFT3.Trading.DoStationCompare()
		end,
		active="YES"
	}

	container = iup.vbox {
		margin="4x4",
		iup.hbox {
			iup.label { title="Station Compare", font=Font.H3 }, iup.fill {},
		},
		iup.fill { size="5" },
		iup.hbox {
			expand="YES",
			TCFT3.Trading.LookupBoughtSold,
			iup.label { title=" Buy in "},
			TCFT3.Trading.StationCompareStation1,
			iup.label { title=" Sell in "},
			TCFT3.Trading.StationCompareStation2,
			TCFT3.Trading.CompareButton,
			TCFT3.Trading.SwapAndCompareButton,
			gap="5",
		},
		iup.fill { size=10 },
		iup.hbox {
			expand="YES",
			TCFT3.Trading.StationCompareList,
		},
	}

	container.tabtitle = "Station Compare"
	container.hotkey = iup.K_s
	function container:OnHelp()
	   TCFT3.ShowHelp("TRADING_STATIONCOMPARE")
	end
	function container:OnActivate(newtab, oldtab)
		if (TCFT3.Connection.isLoggedin==true) then
			TCFT3.SendData({ cmd='450'}) -- Update stationlist
		end
	end
	return container
end


-- Trade routes
local function createTradeRoutesTab()
	local container

	TCFT3.Trading.TradeRouteList = TCFT3.MatrixList:new()
	TCFT3.Trading.TradeRouteList:addcolumn(TCFT3.MatrixList.column.new("itemname", "Item Name", "ALEFT", { "S:A:itemname" }, 21))
	TCFT3.Trading.TradeRouteList:addcolumn(TCFT3.MatrixList.column.new("xp", "XP", "ARIGHT", { "S:D:xp" }, 7, { replace={ ["0"]="--" }, postfix = " xp" }))
	TCFT3.Trading.TradeRouteList:addcolumn(TCFT3.MatrixList.column.new("cuprofit", "Profit / CU", "ARIGHT", { "N:D:cuprofit" }, 9, { postfix = " c", format="%0.1f" }))
	TCFT3.Trading.TradeRouteList:addcolumn(TCFT3.MatrixList.column.new("buylocation", "Buy At", "ALEFT", { "S:A:buylocation" }, 20))
	TCFT3.Trading.TradeRouteList:addcolumn(TCFT3.MatrixList.column.new("selllocation", "Sell at", "ALEFT", { "S:A:selllocation" }, 20))
	TCFT3.Trading.TradeRouteList:addcolumn(TCFT3.MatrixList.column.new("hops", "Hops", "ARIGHT", { "N:A:hops" }, 5))
	TCFT3.Trading.TradeRouteList:addcolumn(TCFT3.MatrixList.column.new("volume", "Volume", "ARIGHT", { "N:A:volume" }, 9, { postfix=" cu" }))
	TCFT3.Trading.TradeRouteList:addcolumn(TCFT3.MatrixList.column.new("mass", "Mass", "ARIGHT", { "N:A:mass" }, 9, { postfix=" Kg" }))
	TCFT3.Trading.TradeRouteList:init()
	TCFT3.Trading.TradeRouteList:set_sortcol(3)

	TCFT3.Trading.TradeRouteStation1 = iup.stationsublist { dropdown='yes', size='%17x', expand='NO', visible_items=15, sort="YES" } -- visible_items=25 to 15 -- tokkana
	TCFT3.Trading.TradeRouteStation2 = iup.stationsublist { dropdown='yes', size='%17x', expand='NO', visible_items=15, sort="YES" } -- visible_items=25 to 15 -- tokkana

	TCFT3.Trading.TradeRouteSearchMinProfit = iup.stationsublist {"4000", "3000", "2000", "1000", "500"; dropdown='yes', expand='NO', sort="NO" }
	TCFT3.Trading.TradeRouteSearchSkipShips = iup.stationtoggle{ value="NO", title="No Ships" }
	TCFT3.Trading.TradeRouteSearchSkipWeps = iup.stationtoggle{ value="NO", title="No Weapons" }
	TCFT3.Trading.TradeRouteSearchSkipBatts = iup.stationtoggle{ value="NO", title="No Batteries" }

	TCFT3.Trading.TradeRouteSearchBtn = iup.stationbutton {
		title="Search for routes",
		action = function(self)
			TCFT3.Trading.TradeRouteSearch()
		end,
		active="YES"
	}
	TCFT3.Trading.SwapRouteAndSearchBtn = iup.stationbutton {
		title="Swap and Search",
		action = function(self)
			local tmp = TCFT3.Trading.TradeRouteStation1.value
			TCFT3.Trading.TradeRouteStation1.value = TCFT3.Trading.TradeRouteStation2.value
			TCFT3.Trading.TradeRouteStation2.value = tmp
			TCFT3.Trading.TradeRouteSearch()
		end,
		active="YES"
	}

	container = iup.vbox {
		margin="4x4",
		iup.hbox {
			iup.label { title="Search for Trade Routes", font=Font.H3*HUD_SCALE }, iup.fill {},
		},
		iup.fill { size="5" },
		iup.hbox {
			expand="YES",
			TCFT3.Trading.LookupBoughtSold,
			iup.label { title=" From "},
			TCFT3.Trading.TradeRouteStation1,
			iup.label { title=" To "},
			TCFT3.Trading.TradeRouteStation2,
			TCFT3.Trading.TradeRouteSearchBtn,
			TCFT3.Trading.SwapRouteAndSearchBtn,
			gap="5",
		},
		iup.fill { size=5 },
		iup.hbox {
			expand="YES",
			margin="5x0",
			iup.label { title="Minimum Profit / CU" },
			iup.fill { size=4 },
			TCFT3.Trading.TradeRouteSearchMinProfit,
			iup.fill { size=10 },
			TCFT3.Trading.TradeRouteSearchSkipShips,
			iup.fill { size=10 },
			TCFT3.Trading.TradeRouteSearchSkipWeps,
			iup.fill { size=10 },
			TCFT3.Trading.TradeRouteSearchSkipBatts,
		},
		iup.fill { size=10 },
		iup.hbox {
			expand="YES",
			TCFT3.Trading.TradeRouteList,
		},
	}

	container.tabtitle = "Trade Routes"
	container.hotkey = iup.K_r
	function container:OnHelp()
	   TCFT3.ShowHelp("TRADING_TRADEROUTES")
	end
	function container:OnActivate(newtab, oldtab)
		if (TCFT3.Connection.isLoggedin==true) then
			TCFT3.SendData({ cmd='450'}) -- Update stationlist
		else
			TCFT3.Trading.TradeRouteSearchUpdateStations()
		end
	end
	return container
end

-- Stations that need update
local function createStationsTab()
	local container

	TCFT3.Trading.StationUpdateList = TCFT3.MatrixList:new()
	TCFT3.Trading.StationUpdateList:addcolumn(TCFT3.MatrixList.column.new("location", "Location", "ALEFT", { "S:A:location" }, 25))
	TCFT3.Trading.StationUpdateList:addcolumn(TCFT3.MatrixList.column.new("stationname", "Station Name", "ALEFT", { "S:A:stationname" }, 25))
	TCFT3.Trading.StationUpdateList:addcolumn(TCFT3.MatrixList.column.new("itemname", "Item Name", "ALEFT", { "S:A:itemname" }, 25))
	TCFT3.Trading.StationUpdateList:addcolumn(TCFT3.MatrixList.column.new("dateupdate", "Last Updated", "ARIGHT", { "S:A:dateupdate" }, 12))
	TCFT3.Trading.StationUpdateList:addcolumn(TCFT3.MatrixList.column.new("username", "Updated By", "ALEFT", { "S:A:username" }, 13))
	TCFT3.Trading.StationUpdateList:set_sortcol(4)
	TCFT3.Trading.StationUpdateList:init()

	container = iup.vbox {
		margin="4x4",
		iup.hbox {
			iup.label { title="Stations that might needs an update", font=Font.H3*HUD_SCALE }, iup.fill { size="10" },
			iup.stationbutton {
				title="Refresh List",
				action = function(self)
					if (TCFT3.Connection.isLoggedin) then
						TCFT3.Trading.FetchUpdateStations()
					else
						TCFT3.ErrorDialog("Stations", "You are not logged in.")
					end
				end,
				active="YES"
			},
		},
		iup.fill { size="5" },
		iup.hbox {
			expand="YES",
			TCFT3.Trading.StationUpdateList,
		},
	}

	container.tabtitle = "Stations"
	container.hotkey = iup.K_i
	function container:OnHelp()
	   TCFT3.ShowHelp("TRADING_STATIONS")
	end
	return container
end


-- Trading tools

local function calculatePrice(resField)
	local itemname = TCFT3.Tools.Inventorylist[TCFT3.Tools.Inventorylist.value] or ""
	if (itemname=="") then
		resField.value="???"
		return
	end
	local itemid = TCFT3.Tools.InventoryHash[itemname]
	if (not itemid) then
		resField.value="???"
		return
	end
	local value
	if (resField==TCFT3.Tools.SellRes1) then
		value = tonumber(TCFT3.Tools.SellCalc1.value) or 0
	else
		value = tonumber(TCFT3.Tools.SellCalc2.value) or 0
	end
	if (value==0) then
		resField.value="???"
		return
	end
	local onepcs = GetStationSellableInventoryPriceByID(itemid, 1)
	local info = GetStationSellableInventoryInfoByID(itemid)
	local costprice = tonumber(info.unitcost)
	local profitonepcs = onepcs - costprice
	local num = 1
	local prev = -99999999
	for i=1, 1001 do
		local curr = GetStationSellableInventoryPriceByID(itemid, i) / i
		if (i==1) and (prev==curr) then -- Static price - can sell unlimited
			resField.value="[S] ~"
			return
		end
		local profit = curr - costprice
		local percent = profit/profitonepcs*100
		if (resField==TCFT3.Tools.SellRes1) then
			if (profit<=value) then
				resField.value = (i-1)
				return
			end
		else
			if (percent<=value) then
				resField.value = (i-1)
				return
			end
		end
		prev = curr
	end
	resField.value = "1000+"
end

TCFT3.Tools.UpdatePriceOnePcs = function()
	local itemname = TCFT3.Tools.Inventorylist[TCFT3.Tools.Inventorylist.value] or ""
	if (itemname=="") then return end
	local itemid = TCFT3.Tools.InventoryHash[itemname]
	if (not itemid) then return end
	local onepcs = GetStationSellableInventoryPriceByID(itemid, 1)
	local info = GetStationSellableInventoryInfoByID(itemid)
	local profit = tonumber(onepcs) - tonumber(info.unitcost)
	local profitcolor = TCFT3.colors.GREEN2
	if (profit<0) then
		profitcolor = TCFT3.colors.RED
	end
	TCFT3.Tools.SellPriceOnePcs.title = onepcs .. "c (profit " .. profitcolor .. profit .. TCFT3.colors.NORMAL .. "c)"
	TCFT3.Tools.SellRes1.value=""
	TCFT3.Tools.SellRes2.value=""
	TCFT3.Tools.SellCalc1.value=""
	TCFT3.Tools.SellCalc2.value=""
end

local function createToolsTab()
	local container


	TCFT3.Tools.Inventorylist = iup.stationsublist { dropdown='yes', size='%17x', expand='NO', visible_items=15, sort="YES", action=TCFT3.Tools.UpdatePriceOnePcs } -- visible_items=25 to 15 -- tokkana
	TCFT3.Tools.SellPriceOnePcs = iup.label { title="", expand="NO", alignment="ALEFT", size="220x" }
	TCFT3.Tools.SellCalc1 = iup.text { value="", expand="NO", size="60x", alignment="ARIGHT" }
	TCFT3.Tools.SellCalc2 = iup.text { value="", expand="NO", size="60x", alignment="ARIGHT" }
	TCFT3.Tools.SellRes1 = iup.text { value="", expand="NO", size="70x", alignment="ARIGHT", readonly="YES", canfocus="NO", border="NO" }
	TCFT3.Tools.SellRes2 = iup.text { value="", expand="NO", size="70x", alignment="ARIGHT", readonly="YES", canfocus="NO", border="NO" }

	container = iup.vbox {
		margin="4x4",
		iup.hbox {
			iup.label { title="Trade Tools", font=Font.H3*HUD_SCALE }, iup.fill {},
		},
		iup.fill { size=10 },
		iup.label { title="Price calculator"},
		iup.fill { size=5 },
		iup.hbox {
			iup.label { title="Item " }, TCFT3.Tools.Inventorylist, iup.fill { size=10 }, iup.label { title="Price when selling 1 pcs: " }, TCFT3.Tools.SellPriceOnePcs,
		},
		iup.fill { size=10 },
		iup.vbox {
			iup.hbox {
				iup.label { title="Numbers to sell to get at least" },
				TCFT3.Tools.SellCalc1,
				iup.label { title="c profit for each item.", size ="180x"},
				iup.stationbutton {
					title="Calculate",
					action = function(self)
						calculatePrice(TCFT3.Tools.SellRes1)
					end,
				},
				iup.label { title="Items to sell: " }, TCFT3.Tools.SellRes1,
				gap=4,
			},
		},
		iup.fill { size=10 },
		iup.vbox {
			iup.hbox {
				iup.label { title="Numbers to sell to get at least" },
				TCFT3.Tools.SellCalc2,
				iup.label { title="% profit of max price", size ="180x"},
				iup.stationbutton {
					title="Calculate",
					action = function(self)
						calculatePrice(TCFT3.Tools.SellRes2)
					end,
				},
				iup.label { title="Items to sell: " }, TCFT3.Tools.SellRes2,
				gap=4,
			},
		},
	}
	container.tabtitle = "Tools"
	container.hotkey = iup.K_o
	function container:OnHelp()
	   TCFT3.ShowHelp("TRADING_TOOLS")
	end
	function container:OnActivate(newtab, oldtab)
		TCFT3.Tools.InventoryHash = {}
		TCFT3.Tools.Inventorylist[1] = nil
		local idx = 1
		for _,info in StationSellableInventoryPairs() do
			if (info.type~=nil) and (TCFT3.Tools.InventoryHash[info.name]==nil) then
				TCFT3.Tools.Inventorylist[idx] = info.name
				TCFT3.Tools.InventoryHash[info.name] = info.itemid
				idx = idx + 1
			end
		end
		TCFT3.Tools.Inventorylist.value=1
		TCFT3.Tools.UpdatePriceOnePcs()
	end

	return container
end

TCFT3.CreateTradeTab = function()
	local ItemLookupTab = createItemlookupTab()
	local StationCompareTab = createStationCompareTab()
	local TradeRoutesTab = createTradeRoutesTab()
	local StationsTab = createStationsTab()
	local ToolsTab = createToolsTab()

    return TCFT3.subtabs { StationCompareTab, ItemLookupTab, TradeRoutesTab, StationsTab, ToolsTab,
    	OnActivate = function(self, newtab, oldtab)
    		if (TCFT3.Connection.isLoggedin==true) then
				TCFT3.SendData({ cmd='450'}) -- Update stationlist
			end
    	end
    }
end
